@@ -1,8 +1,8 @@
 /**************************************************************************
    zfuncs.h      include file for zfuncs functions
 
-   Copyright 2006 2007 2008 2009 2010 2011 2012  Michael Cornelison
-   source URL:  kornelix.squarespace.com
+   Copyright 2006 2007 2008 2009 2010 2011 2012 2013  Michael Cornelison
+   source URL:  kornelix.com
    contact: kornelix2@googlemail.com
    
    This program is free software: you can redistribute it and/or modify
@@ -20,7 +20,7 @@
 
 ***************************************************************************/
 
-//     zfuncs.h   version  v.5.2
+//     zfuncs.h   version  v.5.5
 
 #include <sys/time.h>
 #include <sys/stat.h>
@@ -41,6 +41,9 @@
 #include <locale.h>
 #include <gtk/gtk.h>
 
+#define VERTICAL GTK_ORIENTATION_VERTICAL                                  //  GTK shortcuts
+#define HORIZONTAL GTK_ORIENTATION_HORIZONTAL
+
 #define  int8   char                                                       //  number types
 #define  int16  short
 #define  int32  long
@@ -69,8 +72,9 @@
 //  system functions ======================================================
 
 void apppause();                                                           //  output message and wait for user
-void apppause(cchar * pMess, ... );                                        //  same, works like printf
-void zappcrash(cchar *pMessage, ...);                                      //  crash with popup message in text window
+void apppause(cchar * format, ... );                                       //  same, works like printf
+void zpopup_message(cchar *format, ...);                                   //  popup message, thread safe
+void zappcrash(cchar *format, ...);                                        //  crash with popup message in text window
 void catch_signals();                                                      //  catch signals and do backtrace dump
 void beroot(int argc = 0, char *argv[] = 0);                               //  restart image as root if password OK
 
@@ -82,6 +86,7 @@ double get_CPUtimer(double &time0);                                        //  g
 double CPUtime();                                                          //  get elapsed process CPU time for main()
 double CPUtime2();                                                         //   " include all threads 
 double jobtime();                                                          //   " include all threads + subprocesses
+void compact_time(const time_t fileDT, char *compactDT);                   //  time_t date/time to yyyymmddhhmmss
 int  parseprocfile(cchar *pfile, cchar *pname, double *value, ...);        //  get data from /proc file
 int  parseprocrec(char *prec, int field, double *value, ...);              //  get data from /proc file record
 void zsleep(double dsecs);                                                 //  sleep specified seconds
@@ -100,11 +105,15 @@ void gate_threads_leave();                                                 //
 
 char * zmalloc(size_t bytes, cchar *tag = 0);                              //  malloc() with auto crash and log option
 void zfree(void *buff);                                                    //  free() with log option
-void zmalloc_log(int count);                                               //  turn zmalloc() log on/off
 void zmalloc_report();                                                     //  print memory allocation report
 
+int shell_quiet(cchar *command, ...);                                      //  format/run shell command, return status
+int shell_ack(cchar *command, ...);                                        //   ""  + popup an error message if error
+int shell_asynch(cchar *command, ...);                                     //  start shell command, return immediately
+int shell_asynch_status(int handle);                                       //  get status of asynch shell command
 char * command_output(int &contx, cchar *command, ...);                    //  get shell command output
 int command_status(int contx);                                             //  get exit status of command
+int command_kill(int contx);                                               //  kill command before completion
 int signalProc(cchar * pname, cchar * signal);                             //  pause/resume/kill subprocess
 int runroot(cchar *sucomm, cchar *command);                                //  run command as root via su or sudo
 int checkinstall(cchar *prog, ...);                                        //  check if programs are installed
@@ -165,6 +174,41 @@ int  utf8substring(char *utf8out, cchar *utf8in, int pos, int cc);         //  g
 int  utf8_check(cchar *string);                                            //  check utf8 string for encoding errors
 int  utf8_position(cchar *utf8in, int Nth);                                //  get byte position of Nth graphic char.
 
+//  number conversion =====================================================
+
+int convSI (cchar *inp, int &inum, cchar **delm = 0);                            //  string to int
+int convSI (cchar *inp, int &inum, int low, int hi, cchar **delm = 0);           //  (with low/high limit checking)
+int convSD (cchar *inp, double &dnum, cchar **delm = 0);                         //  string to double
+int convSD (cchar *inp, double &dnum, double low, double hi, cchar **delm = 0);  //  (with low/high limit checking)
+int convSF (cchar *inp, float &fnum, cchar **delm = 0);                          //  string to double
+int convSF (cchar *inp, float &fnum, float low, float hi, cchar **delm = 0);     //  (with low/high limit checking)
+int convIS (int iin, char *outp, int *cc = 0);                                   //  int to string, returned cc
+int convDS (double din, int prec, char *outp, int *cc = 0);                      //  double to string, precision, ret. cc
+char * formatKBMB(double fnum, int prec);                                        //  format nnn B, nn.n KB, n.nn MB, etc.
+
+//  wildcard functions ====================================================
+
+int MatchWild(cchar * wildstr, cchar * str);                               //  wildcard string match (match = 0)
+int MatchWildIgnoreCase(cchar * wildstr, cchar * str);                     //  wildcard string match, ignoring case
+cchar * SearchWild(cchar *wpath, int &flag);                               //  wildcard file search
+cchar * SearchWildIgnoreCase(cchar *wpath, int &flag);                     //  wildcard file search, ignoring case
+
+//  search and sort functions =============================================
+
+int bsearch(int seekint, int nn, int list[]);                              //  binary search sorted list[nn]
+int bsearch(char *seekrec, char *allrecs, int recl, int nrecs);            //  binary search sorted records
+int bsearch(char *seekrec, char **allrecs, int N, int nrecs);              //  binary search of sorted pointers to recs
+
+typedef int HeapSortUcomp(cchar *rec1, cchar *rec2);                       //  return -1/0/+1 if rec1 </=/> rec2
+void HeapSort(int vv[], int nn);                                           //  Heap Sort - integer
+void HeapSort(float vv[], int nn);                                         //  Heap Sort - float
+void HeapSort(double vv[], int nn);                                        //  Heap Sort - double
+void HeapSort(char *vv[], int nn);                                         //  Heap Sort - char *, ascending order
+void HeapSort(char *vv[], int nn, HeapSortUcomp);                          //  Heap Sort - char *, user-defined order 
+void HeapSort(char *recs, int RL, int NR, HeapSortUcomp);                  //  Heap Sort - records, user-defined order
+
+int MemSort(char * RECS, int RL, int NR, int KEYS[][3], int NK);           //  memory sort, records with multiple keys
+
 //  bitmap functions ======================================================
 
 struct bitmap {
@@ -197,39 +241,6 @@ int pvlist_replace(pvlist *pv, int Nth, cchar *entry);                     //  r
 char * pvlist_get(pvlist *pv, int Nth);                                    //  return Nth entry (0...)
 int pvlist_sort(pvlist *pv);                                               //  sort list, ascending
 
-//  number conversion =====================================================
-
-int convSI (cchar *inp, int &inum, cchar **delm = 0);                            //  string to int
-int convSI (cchar *inp, int &inum, int low, int hi, cchar **delm = 0);           //  (with low/high limit checking)
-int convSD (cchar *inp, double &dnum, cchar **delm = 0);                         //  string to double
-int convSD (cchar *inp, double &dnum, double low, double hi, cchar **delm = 0);  //  (with low/high limit checking)
-int convSF (cchar *inp, float &fnum, cchar **delm = 0);                          //  string to double
-int convSF (cchar *inp, float &fnum, float low, float hi, cchar **delm = 0);     //  (with low/high limit checking)
-int convIS (int iin, char *outp, int *cc = 0);                                   //  int to string, returned cc
-int convDS (double din, int prec, char *outp, int *cc = 0);                      //  double to string, precision, ret. cc
-char * formatKBMB(double fnum, int prec);                                        //  format nnn B, nn.n KB, n.nn MB, etc.
-
-//  wildcard functions ====================================================
-
-int MatchWild(cchar * wildstr, cchar * str);                               //  wildcard string match (0)
-cchar * SearchWild(cchar *wpath, int &flag);                               //  wildcard file search
-
-//  search and sort functions =============================================
-
-int bsearch(int seekint, int nn, int list[]);                              //  binary search sorted list[nn]
-int bsearch(char *seekrec, char *allrecs, int recl, int nrecs);            //  binary search sorted records
-int bsearch(char *seekrec, char **allrecs, int N, int nrecs);              //  binary search of sorted pointers to recs
-
-typedef int HeapSortUcomp(cchar * rec1, cchar * rec2);                     //  return -1/0/+1 if rec1 </=/> rec2
-void HeapSort(int vv[], int nn);                                           //  Heap Sort - integer
-void HeapSort(float vv[], int nn);                                         //  Heap Sort - float
-void HeapSort(double vv[], int nn);                                        //  Heap Sort - double
-void HeapSort(char * vv[], int nn);                                        //  Heap Sort - char *, ascending order
-void HeapSort(char * vv[], int nn, HeapSortUcomp);                         //  Heap Sort - char *, user-defined order 
-void HeapSort(char * recs, int RL, int NR, HeapSortUcomp);                 //  Heap Sort - records, user-defined order
-
-int MemSort(char * RECS, int RL, int NR, int KEYS[][3], int NK);           //  memory sort, records with multiple keys
-
 //  random number functions ===============================================
 
 int lrandz(int64 * seed);                                                  //  returns 0 to 0x7fffffff
@@ -254,15 +265,13 @@ cchar * get_zprefix();                                                     //  g
 cchar * get_zuserdir();                                                    //  get /home/user/.appname/
 cchar * get_zdatadir();                                                    //  get install directory
 cchar * get_zdocdir();                                                     //  get document directory 
+int  locale_filespec(cchar *ftype, cchar *fname, char *filespec);          //  get a locale dependent file        v.5.5
 
 void showz_userguide(cchar *context = 0);                                  //  show user guide in new process
-void showz_readme();                                                       //  show README file in popup window
-void showz_changelog();                                                    //  show changelog file in popup window
 void showz_logfile();                                                      //  show log file in popup window      v.5.2
-void showz_translations();                                                 //  show TRANSLATIONS file in popup window
-void showz_doctext(cchar *file);                                           //  show text file (or .gz) in a popup window
-void zmake_menu_launcher(cchar *command, cchar *cats, cchar *generic);     //  add desktop menu and launcher      v.4.1
+void showz_textfile(cchar *type, cchar *file);                             //  show text file [.gz] in popup window
 void showz_html(cchar *url);                                               //  show html via preferred browser
+void zmake_menu_launcher(cchar *command, cchar *cats, cchar *generic);     //  add desktop menu and launcher      v.4.1
 
 /**************************************************************************
    GTK utility functions
@@ -272,9 +281,10 @@ void zmainloop(int skip = 0);                                              //  d
 void zthreadcrash();                                                       //  crash if thread is not main() thread
 
 //  text window print and read utilities
+
 void wprintx(GtkWidget *Win, int line, cchar *mess, cchar *font = 0);      //  write text to line, optional font
-void wprintf(GtkWidget *Win, int line, cchar *mess, ...);                  //  "printf" version
-void wprintf(GtkWidget *Win, cchar *mess, ... );                           //  "printf" to next line, scroll up
+void wprintf(GtkWidget *Win, int line, cchar *format, ...);                //  "printf" version
+void wprintf(GtkWidget *Win, cchar *format, ... );                         //  "printf" to next line, scroll up
 void wscroll(GtkWidget *mLog, int line);                                   //  scroll window to put line on screen
 void wclear(GtkWidget *Win);                                               //  clear window
 void wclear(GtkWidget *Win, int line);                                     //  clear from line to end
@@ -283,6 +293,13 @@ int  wfiledump(GtkWidget *Win, char *filespec);                            //  d
 void wfilesave(GtkWidget *Win);                                            //  wfiledump() via file-chooser dialog
 void wprintp(GtkWidget *Win);                                              //  print text window to default printer
 
+//  intercept text window mouse click functions
+
+typedef void clickfunc_t(GtkWidget *widget, int &line, int &pos);          //  function to get clicked text position
+void textwidget_set_clickfunc(GtkWidget *widget, clickfunc_t clickfunc);   //  (wrapped lines are one logical line)
+char * textwidget_get_line(GtkWidget *widget, int line);                   //  get entire line at clicked position
+char * textwidget_get_word(char *line, int pos, cchar *dlims, char &end);  //  get delimited word at clicked position
+
 /**************************************************************************/
 
 //  functions to simplify building menus, tool bars, status bars
@@ -296,18 +313,56 @@ typedef void mtFunc(GtkWidget *, cchar *mname);
 
 GtkWidget * create_menubar(GtkWidget *vbox);                                     //  create menubar in packing box
 GtkWidget * add_menubar_item(GtkWidget *mbar, cchar *mname, mtFunc func = 0);    //  add menu item to menubar 
-GtkWidget * add_submenu_item(GtkWidget *mitem, cchar *label, mtFunc func = 0);   //  add submenu item to menu item 
+GtkWidget * add_submenu_item(GtkWidget *mitem, cchar *subname,                   //  add submenu item to menu item 
+                             mtFunc func = 0, cchar *mtip = 0);                  //    with opt. function and popup tip
 
-GtkWidget * create_popmenu();                                                    //  create an empty popup menu
-GtkWidget * add_popmenu_item(GtkWidget *popmenu, cchar *mname, mtFunc func);     //  add menu item to popup menu
-void popup_menu(GtkWidget *popmenu);                                             //  pop-up menu at current mouse position
+GtkWidget * create_toolbar(GtkWidget *vbox, int iconsize = 24);            //  toolbar in packing box (no vert gtk3)
+GtkWidget * add_toolbar_button(GtkWidget *tbar, cchar *name, cchar *ttip,  //  add button with stock (gtk-quit) 
+                                 cchar *icon, mtFunc func);                //    or custom icon (iconfile.png)
 
-GtkWidget * create_toolbar(GtkWidget *vbox, int iconsize = 24);                  //  toolbar in packing box (no vert gtk3)
-GtkWidget * add_toolbar_button(GtkWidget *tbar, cchar *label, cchar *tip,        //  add button with stock (gtk-quit) 
-                                 cchar *icon, mtFunc func);                      //    or custom icon (iconfile.png)
+GtkWidget * create_stbar(GtkWidget *vbox);                                 //  create status bar in packing box
+int stbar_message(GtkWidget *stbar, cchar *message);                       //  display message in status bar
 
-GtkWidget * create_stbar(GtkWidget *vbox);                                       //  create status bar in packing box
-int stbar_message(GtkWidget *stbar, cchar *message);                             //  display message in status bar
+/**************************************************************************/
+
+GtkWidget * create_popmenu();                                              //  create an empty popup menu
+GtkWidget * add_popmenu_item(GtkWidget *popmenu, cchar *mname,             //  add menu item to popup menu
+                           mtFunc func, cchar *arg, cchar *mtip = 0);
+void popup_menu(GtkWidget *, GtkWidget *popmenu);                          //  pop-up menu at current mouse posn.
+
+/**************************************************************************/
+
+//  user editable graphic menu in popup window
+//  menus can be added and arranged using the mouse
+
+typedef void gmenuz_cbfunc(cchar *menu);                                   //  caller-supplied callback function
+void gmenuz(GtkWidget *parent, cchar *ufile, gmenuz_cbfunc);               //  show window, handle mouse drag/click
+
+/**************************************************************************/
+
+//  create vertical menu/toolbar in vertical packing box                   //  v.5.5
+
+struct vbmenuent {                                                         //  menu data from caller
+   cchar       *name;                                                      //  menu name
+   cchar       *icon;                                                      //  opt. icon file name
+   cchar       *desc;                                                      //  description (mouse hover popup)
+   mtFunc      *func;                                                      //  callback func (GtkWidget *, cchar *arg)
+   cchar       *arg;                                                       //  callback arg
+   GdkPixbuf   *pixbuf1;                                                   //  icon pixbuf or null
+   GdkPixbuf   *pixbuf2;                                                   //  darkened pixbuf (clicked)
+   int         ypos, yhh;                                                  //  layout position, extent
+};
+
+struct vbox_menu {
+   GtkWidget   *vbox;                                                      //  parent window
+   GtkWidget   *layout;                                                    //  drawing window
+   int         mcount;                                                     //  menu entry count
+   vbmenuent   menu[100];
+};
+
+vbox_menu *vbox_menu_new(GtkWidget *vbox);                                 //  create new menu in parent vbox
+void vbox_menu_add(vbox_menu *vbm, cchar *name, cchar *icon,               //  add menu item with response function
+                         cchar *desc, mtFunc func, cchar *arg);            //  function may be popup_menu() 
 
 /**************************************************************************/
 
@@ -315,7 +370,7 @@ int stbar_message(GtkWidget *stbar, cchar *message);
 //   widget types: dialog, hbox, vbox, hsep, vsep, frame, scrwin, label, entry, edit, radio, 
 //                 check, button, togbutt, spin, combo, comboE, hscale, vscale, colorbutt
 
-#define zdmaxwidgets 300
+#define zdmaxwidgets 100                           //  v.5.3
 #define zdmaxbutts 10
 #define zdsentinel 2138687954
 
@@ -340,8 +395,8 @@ struct zdialog  {
       int         zstat;                           //  dialog status (from completion button)
       int         disabled;                        //  widget signals/events are disabled
       int         saveposn;                        //  save and recall window position each use
-      int         stopKB;                          //  next KB event will be ignored
-      cchar       *help_topic;                     //  optional help topic
+      int         saveinputs;                      //  save and recall user inputs each use
+      int         stopKB;                          //  flag, next KB event will be ignored
       GtkWidget   *parent;                         //  parent window or null
       GtkWidget   *compbutt[zdmaxbutts];           //  dialog completion buttons
       zwidget     widget[zdmaxwidgets];            //  dialog widgets (EOF = type = 0)
@@ -359,15 +414,14 @@ int zdialog_add_widget(zdialog *zd,                                        //  a
       cchar *data, cchar *options);                                        //  "scc=nn|homog|expand|space=nn|wrap"
 
 GtkWidget * zdialog_widget(zdialog *zd, cchar *name);                      //  GTK widget from zdialog widget name
-int zdialog_set_group(zdialog *zd, cchar *radio1, ...);                    //  set group ID for set of radio buttons
 int zdialog_resize(zdialog *zd, int width, int height);                    //  set size > widget sizes
 int zdialog_put_data(zdialog *zd, cchar *name, cchar *data);               //  put data in widget (entry, spin ...)
 cchar * zdialog_get_data(zdialog *zd, cchar *name);                        //  get widget data
 int zdialog_set_limits(zdialog *zd, cchar *name, double min, double max);  //  set new widget limits (spin, scale)
-void zdialog_help(zdialog *zd, cchar *help_topic);                         //  set help topic for zdialog
 
 typedef int zdialog_event(zdialog *zd, cchar *name);                       //  widget event callback function
 int zdialog_run(zdialog *zd, zdialog_event = 0, cchar *posn = 0);          //  run dialog, handle events
+void KBstate(int key, int state);                                          //  extern: pass KB events to main app
 
 int zdialog_send_event(zdialog *zd, cchar *event);                         //  send an event to an active dialog
 int zdialog_send_response(zdialog *zd, int zstat);                         //  complete a dialog, set status
@@ -398,6 +452,10 @@ int  zdialog_positions(cchar *action);                                     //  l
 void zdialog_set_position(zdialog *zd, cchar *posn);                       //  set initial/new zdialog window position
 void zdialog_save_position(zdialog *zd);                                   //  save zdialog window position
 
+int zdialog_inputs(cchar *action);                                         //  load or save zdialog input fields
+int zdialog_save_inputs(zdialog *zd);                                      //  save zdialog input fields when finished
+int zdialog_restore_inputs(zdialog *zd);                                   //  restore zdialog inputs from prior use
+
 /**************************************************************************/
 
 //  translation functions
@@ -405,32 +463,33 @@ void zdialog_save_position(zdialog *zd);                                   //  s
 void ZTXinit(cchar *lang);                                                 //  setup for message translation
 cchar *ZTX(cchar *english);                                                //  get translation for English message
 void ZTX_translation_start(GtkWidget *parent);                             //  start online translation
+int ZTXnewtranslations();                                                  //  new translations were added
 
 //  write text to popup window, shell command to popup window
 
-int write_popup_text(cchar *action, cchar *text = 0, int ww = 0, int hh = 0, GtkWidget *parent = 0);
+GtkWidget * write_popup_text(cchar *action, cchar *text = 0, int ww = 0, int hh = 0, GtkWidget *parent = 0);
 int popup_command(cchar *command, int ww = 400, int hh = 300, GtkWidget *parent = 0);
 
 //  popup message dialogs
 
-void zmessageACK(GtkWidget *parent, cchar *pMess, ... );                   //  display message, wait for OK
-void zmessLogACK(GtkWidget *parent, cchar *pMess, ... );                   //  same, with log to STDOUT
-int zmessageYN(GtkWidget *parent, cchar *pMess, ... );                     //  display message, wait for YES or NO
-void zmessage_help(GtkWidget *parent, cchar *topic, cchar *mess, ...);     //  message box, [help] > user guide topic
-zdialog * zmessage_post(GtkWidget *parent, int secs, cchar *mess, ...);    //  display message until timeout or cancel
-char * zdialog_text(GtkWidget *parent, cchar * title, cchar * initext);    //  get short text input from user
-int zdialog_choose(cchar *title, GtkWidget *parent, cchar *mess, ...);     //  show message and return choice button
+void zmessageACK(GtkWidget *parent, cchar *title, cchar *format, ... );             //  display message, wait for OK
+void zmessLogACK(GtkWidget *parent, cchar *format, ... );                           //  same, with log to STDOUT
+int zmessageYN(GtkWidget *parent, cchar *format, ... );                             //  display message, wait for YES/NO
+void zmessage_help(GtkWidget *parent, cchar *topic, cchar *format, ...);            //  message box, user guide topic
+zdialog * zmessage_post(GtkWidget *parent, int secs, cchar *format, ...);           //  show message, timeout or cancel
+char * zdialog_text(GtkWidget *parent, cchar * title, cchar * initext);             //  get short text input from user
+int zdialog_choose(cchar *title, GtkWidget *parent, cchar *format, ...);            //  show message, return choice 
+void poptext_window(cchar *text, GtkWindow *pwin, int mx, int my, int secs = 0);    //  show popup text at window posn
+void poptext_mouse(GdkDevice *mouse, cchar *text, int dx, int dy, int secs = 0);    //  show popup text at mouse posn
 
 //  file chooser dialogs for one file or multiple files
 
 char  * zgetfile1(cchar *title, cchar *action, cchar *file, cchar *butt = 0);
 char ** zgetfileN(cchar *title, cchar *action, cchar *file, cchar *butt = 0);
 
-//  print image file
+//  print an image file, choosing printer, paper, orientation, margins, and scale
 
-void print_image_paper_setup(GtkWidget *parent);                           //  paper setup for printing an image file
-void print_image_margins_setup(GtkWidget *parent);                         //  setup print margins
-void print_image_file(cchar *imagefile);                                   //  print the image file
+void print_image_file(GtkWidget *parent, cchar *imagefile);
 
 //  drag and drop functions
 
